__all__ = ()

IDENTIFIER_NONE = 0
IDENTIFIER_TESTING_START = 1
IDENTIFIER_TESTING_END = 2
IDENTIFIER_FILE_REGISTRATION = 3
IDENTIFIER_FILE_REGISTRATION_DONE = 4
IDENTIFIER_FILE_LOAD_DONE = 5
IDENTIFIER_FILE_TESTING_DONE = 6
IDENTIFIER_TEST_DONE = 7
IDENTIFIER_SOURCE_LOAD_FAILURE = 8
